@@ -8,6 +8,7 @@
 # - Apache 2.0
 # - BSD 2-Clause
 # - BSD 3-Clause
+# - BSD 4-Clause
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -43,7 +44,7 @@ class ConfigParser(ArgumentParser):  # pylint: disable=C0115
     which has strict checking on command line arguments.
     """
 
-    def parse(
+    def parse(  # pylint: disable=R0912
         self,
         args: Optional[Sequence[str]] = None,
         config: Optional[Config] = None,
@@ -109,6 +110,8 @@ class ConfigParser(ArgumentParser):  # pylint: disable=C0115
             if arg.startswith("--"):
                 key_value_args.append(arg.split("=", maxsplit=1))
             else:
+                if not key_value_args:
+                    continue
                 key_value_args[-1].append(arg)
         for key_value in key_value_args:
             if key_value[0] not in self._option_string_actions:
@@ -197,7 +200,7 @@ class Config(NestedDict):
     >>> c.i.d
     1013
     >>> c.d.i
-    Config()
+    Config(<class 'chanfig.config.Config'>, )
     >>> c.freeze().dict()
     {'f': {'n': 'chang'}, 'i': {'d': 1013}, 'd': {'i': {}}}
     >>> c.d.i = 1013
@@ -247,8 +250,8 @@ class Config(NestedDict):
         ...         return self
         >>> c = PostConfig(data="path")
         >>> c.post()
-        PostConfig(
-          ('data'): Config(
+        PostConfig(<class 'chanfig.config.Config'>,
+          ('data'): Config(<class 'chanfig.config.Config'>,
             ('feature'): 'path'
             ('label'): 'path'
           )
@@ -454,17 +457,20 @@ class Config(NestedDict):
         >>> d.get('f', 2)
         2
         >>> d.f
-        Config()
+        Config(<class 'chanfig.config.Config'>, )
         >>> del d.f
         >>> d.freeze()
-        Config(
-          ('i'): Config(
+        Config(<class 'chanfig.config.Config'>,
+          ('i'): Config(<class 'chanfig.config.Config'>,
             ('d'): 1013
           )
         )
         >>> d.f
         Traceback (most recent call last):
         KeyError: 'f'
+        >>> d["f.n"]
+        Traceback (most recent call last):
+        KeyError: 'f.n'
 
         ```
         """
@@ -549,12 +555,12 @@ class Config(NestedDict):
         >>> "i.d" in d
         False
         >>> d.i.d
-        Config()
+        Config(<class 'chanfig.config.Config'>, )
         >>> "i.d" in d
         True
         >>> del d.f.n
         >>> d.f.n
-        Config()
+        Config(<class 'chanfig.config.Config'>, )
         >>> del d.c
         Traceback (most recent call last):
         KeyError: 'c'
