@@ -8,6 +8,7 @@
 # - Apache 2.0
 # - BSD 2-Clause
 # - BSD 3-Clause
+# - BSD 4-Clause
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -19,6 +20,7 @@ from functools import wraps
 from os import PathLike
 from typing import TYPE_CHECKING, Any, Callable, Iterable, Iterator, Mapping, Optional, Tuple, Union
 
+from .default_dict import DefaultDict
 from .flat_dict import FlatDict, PathStr
 from .utils import Null
 from .variable import Variable
@@ -28,27 +30,22 @@ if TYPE_CHECKING:
     from torch import dtype as TorchDtype
 
 
-class NestedDict(FlatDict):
+class NestedDict(DefaultDict):
     r"""
-    `NestedDict` further extends `FlatDict` object by introducing a nested structure with `delimiter`.
+    `NestedDict` further extends `DefaultDict` object by introducing a nested structure with `delimiter`.
     By default, `delimiter` is `.`, but it could be modified in subclass or by calling `dict.setattr('delimiter', D)`.
 
     `d = NestedDict({"a.b.c": 1})` is equivalent to `d = NestedDict({"a": {"b": {"c": 1}}})`,
-    and you can access the members either by `d["a.b.c"]` or more simply by `d.a.b.c`.
+    and you can access members either by `d["a.b.c"]` or more simply by `d.a.b.c`.
 
     This behavior allows you to pass keyword arguments to other function as easy as `func1(**d.func1)`.
 
-    `NestedDict` supports `default_factory` in the same manner as `collections.defaultdict`.
-    It `default_factory is not None`, the value will be set to `default_factory()`
-    when you access a key that does not exist in `NestedDict`.
+    Since `NestedDict` inherits from `DefaultDict`, it also supports `default_factory`.
     With `default_factory`, you can assign `d.a.b.c = 1` without assign `d.a = NestedDict()` in the first place.
+    Note that the constructor of `NestedDict` is different from `DefaultDict`, `default_factory` is not a positional
+    argument, and must be set in a keyword argument.
 
-    You may specify `default_factory=NestedDict` when creating the `NestedDict` or
-    by calling `dict.setattr('default_factory', NestedD`ict)`.
-
-    Note that just like `collections.defaultdict`, `default_factory()` is called without any arguments.
-
-    `NestedDict` also has `all_keys`, `all_values`, `all_items` methods to get all keys, values, items
+    `NestedDict` also introduce `all_keys`, `all_values`, `all_items` methods to get all keys, values, items
     respectively in the nested structure.
 
     Attributes:
@@ -60,10 +57,10 @@ class NestedDict(FlatDict):
             Delimiter for nested structure.
 
     Notes:
-        When `convert_mapping` specified, all new values with a type of `Mapping`
+        When `convert_mapping` specified, all new values with type of `Mapping`
         will be converted to `default_mapping`.
 
-        `convert_mapping` is automatically applied to arguments at initialisation.
+        `convert_mapping` is automatically applied to arguments during initialisation.
 
     Examples:
     ```python
@@ -83,6 +80,9 @@ class NestedDict(FlatDict):
     convert_mapping: bool = False
     delimiter: str = "."
 
+    def __init__(self, *args, default_factory: Optional[Callable] = None, **kwargs) -> None:
+        super().__init__(default_factory, *args, **kwargs)
+
     def _init(self, *args, **kwargs) -> None:
         if len(args) == 1:
             args = args[0]
@@ -242,23 +242,29 @@ class NestedDict(FlatDict):
         >>> d.get('f', 2)
         2
         >>> d.f
-        NestedDict()
+        NestedDict(<class 'chanfig.nested_dict.NestedDict'>, )
         >>> del d.f
         >>> d = NestedDict()
-        >>> d.f
+        >>> d.e
         Traceback (most recent call last):
-        KeyError: 'f'
+        KeyError: 'e'
+        >>> d.e.f
+        Traceback (most recent call last):
+        KeyError: 'e'
 
         ```
         """
 
         delimiter = self.getattr("delimiter", ".")
-        while isinstance(name, str) and delimiter in name:
-            name, rest = name.split(delimiter, 1)
-            self, name = self[name], rest  # pylint: disable=W0642
+        try:
+            while isinstance(name, str) and delimiter in name:
+                name, rest = name.split(delimiter, 1)
+                self, name = self[name], rest  # pylint: disable=W0642
+        except (AttributeError, TypeError):
+            raise KeyError(name) from None
         if not isinstance(self, NestedDict):
-            if default is not Null:
-                return self[name] if name in self else default
+            if default is not Null and name not in self:
+                return default
             return self[name]
         return super().get(name, default)
 
@@ -299,6 +305,12 @@ class NestedDict(FlatDict):
         >>> d.n.l = 'liu'
         >>> d['n.l']
         'liu'
+        >>> d['f.n.e'] = "error"
+        Traceback (most recent call last):
+        ValueError: Cannot set `f.n.e` to `error`, as `f.n=chang`.
+        >>> d['f.n.e.a'] = "error"
+        Traceback (most recent call last):
+        KeyError: 'e'
         >>> d.setattr('convert_mapping', True)
         >>> d.a.b = {'c': {'d': 1}, 'e.f' : 2}
         >>> d.a.b.c.d
@@ -313,20 +325,29 @@ class NestedDict(FlatDict):
         default_mapping = self.getattr("default_mapping", NestedDict)
         if convert_mapping is None:
             convert_mapping = self.convert_mapping
-        while isinstance(name, str) and delimiter in name:
-            name, rest = name.split(delimiter, 1)
-            if name not in self:
-                if convert_mapping:
-                    super().__setitem__(name, default_mapping())
-                else:
-                    self.__missing__(name)
-            self, name = self[name], rest  # pylint: disable=W0642
+        full_name = name
+        try:
+            while isinstance(name, str) and delimiter in name:
+                name, rest = name.split(delimiter, 1)
+                if name not in self:
+                    if convert_mapping:
+                        super().__setitem__(name, default_mapping())
+                    else:
+                        self.__missing__(name)
+                self, name = self[name], rest  # pylint: disable=W0642
+        except (AttributeError, TypeError):
+            raise KeyError(name) from None
         if convert_mapping and isinstance(value, Mapping):
             value = default_mapping(value)
-        if not isinstance(self, NestedDict):
-            self[name] = value
+        if isinstance(self, Mapping):
+            if not isinstance(self, NestedDict):
+                self[name] = value
+            else:
+                super().__setitem__(name, value)
         else:
-            super().__setitem__(name, value)
+            raise ValueError(
+                f"Cannot set `{full_name}` to `{value}`, as `{delimiter.join(full_name.split(delimiter)[:-1])}={self}`."
+            )
 
     __setitem__ = set
     __setattr__ = set
@@ -360,22 +381,72 @@ class NestedDict(FlatDict):
         >>> d.f.n
         Traceback (most recent call last):
         KeyError: 'n'
-        >>> del d.c
+        >>> del d.e
         Traceback (most recent call last):
-        KeyError: 'c'
+        KeyError: 'e'
+        >>> del d.e.f
+        Traceback (most recent call last):
+        KeyError: 'f'
 
         ```
         """
 
         delimiter = self.getattr("delimiter", ".")
-        while isinstance(name, str) and delimiter in name:
-            name, rest = name.split(delimiter, 1)
-            self, name = self[name], rest  # pylint: disable=W0642
+        try:
+            while isinstance(name, str) and delimiter in name:
+                name, rest = name.split(delimiter, 1)
+                self, name = self[name], rest  # pylint: disable=W0642
+        except (AttributeError, TypeError):
+            raise KeyError(name) from None
         super().__delitem__(name)
 
     __delitem__ = delete
     __delattr__ = delete
 
+    def pop(self, name: str, default: Any = Null) -> Any:
+        r"""
+        Pop value from `NestedDict`.
+
+        Args:
+            name:
+            default:
+
+        Returns:
+            value: If `NestedDict` does not contain `name`, return `default`.
+
+        Examples:
+        ```python
+        >>> d = NestedDict({"i.d": 1013, "f.n": "chang", "n.a.b.c": 1}, default_factory=NestedDict)
+        >>> d.pop('i.d')
+        1013
+        >>> d.pop('i.d', True)
+        True
+        >>> d.pop('i.d')
+        Traceback (most recent call last):
+        KeyError: 'd'
+        >>> d.pop('e')
+        Traceback (most recent call last):
+        KeyError: 'e'
+        >>> d.pop('e.f')
+        Traceback (most recent call last):
+        KeyError: 'f'
+
+        ```
+        """
+
+        delimiter = self.getattr("delimiter", ".")
+        try:
+            while isinstance(name, str) and delimiter in name:
+                name, rest = name.split(delimiter, 1)
+                self, name = self[name], rest  # pylint: disable=W0642
+        except (AttributeError, TypeError):
+            raise KeyError(name) from None
+        if not isinstance(self, dict) or name not in self:
+            if default is not Null:
+                return default
+            raise KeyError(name)
+        return super().pop(name)
+
     def dict(self, cls: Callable = dict) -> Mapping:
         r"""
         Convert `NestedDict` to other `Mapping`.
@@ -538,42 +609,6 @@ class NestedDict(FlatDict):
 
         return self.apply(lambda _: super().to(cls))
 
-    def pop(self, name: str, default: Any = Null) -> Any:
-        r"""
-        Pop value from `NestedDict`.
-
-        Args:
-            name:
-            default:
-
-        Returns:
-            value: If `NestedDict` does not contain `name`, return `default`.
-
-        Examples:
-        ```python
-        >>> d = NestedDict({"i.d": 1013, "f.n": "chang"}, default_factory=NestedDict)
-        >>> d.pop('i.d')
-        1013
-        >>> d.pop('i.d', True)
-        True
-        >>> d.pop('i.d')
-        Traceback (most recent call last):
-        KeyError: 'd'
-        >>> d.pop('n.l')
-        Traceback (most recent call last):
-        KeyError: 'n'
-
-        ```
-        """
-
-        delimiter = self.getattr("delimiter", ".")
-        if delimiter in name:
-            name, rest = name.split(delimiter, 1)
-            if name not in self:
-                raise KeyError(name)
-            return self[name].pop(rest, default)
-        return super().pop(name, default) if default is not Null else super().pop(name)
-
     def __contains__(self, name: Any) -> bool:  # type: ignore
         delimiter = self.getattr("delimiter", ".")
         try:
@@ -583,14 +618,3 @@ class NestedDict(FlatDict):
             return super().__contains__(name)
         except (TypeError, KeyError):  # TypeError when name is not in self
             return False
-
-
-class DefaultDict(NestedDict):
-    r"""
-    `NestedDict` with `default_factory` set to `NestedDict` by default.
-    """
-
-    def __init__(self, *args, **kwargs):
-        if "default_factory" not in kwargs:
-            kwargs["default_factory"] = NestedDict
-        super().__init__(*args, **kwargs)
